=== modified file 'NEWS'
--- NEWS	2005-12-15 03:29:58 +0000
+++ NEWS	2005-12-15 08:32:03 +0000
@@ -95,6 +95,9 @@
     * 'bzr add' now takes a --dry-run parameter which shows you what would be
       added, but doesn't actually add anything. (Michael Ellerman)
 
+    * bzr add now lists how many files were ignored per glob.  add --verbose
+      lists the specific files.  (Aaron Bentley)
+
   BUG FIXES:
 
     * SFTP can walk up to the root path without index errors. (Robert Collins)

=== modified file 'bzrlib/add.py'
--- bzrlib/add.py	2005-12-15 03:21:11 +0000
+++ bzrlib/add.py	2005-12-15 08:32:03 +0000
@@ -100,7 +100,8 @@
     file_list = _prepare_file_list(file_list)
     user_list = file_list[:]
     inv = tree.read_working_inventory()
-    count = 0
+    added = []
+    ignored = {}
 
     for f in file_list:
         rf = tree.relpath(f)
@@ -146,26 +147,31 @@
         elif sub_tree:
             mutter("%r is a bzr tree", f)
         else:
-            count += __add_one(tree, inv, rf, kind, action)
+            added.extend(__add_one(tree, inv, rf, kind, action))
 
         if kind == 'directory' and recurse and not sub_tree:
             for subf in os.listdir(af):
                 subp = os.path.join(rf, subf)
                 if subf == bzrlib.BZRDIR:
                     mutter("skip control directory %r", subp)
-                elif tree.is_ignored(subp):
-                    mutter("skip ignored sub-file %r", subp)
                 else:
-                    mutter("queue to add sub-file %r", subp)
-                    file_list.append(tree.abspath(subp))
-
-
-    mutter('added %d entries', count)
+                    ignore_glob = tree.is_ignored(subp)
+                    if ignore_glob is not None:
+                        mutter("skip ignored sub-file %r", subp)
+                        if ignore_glob not in ignored:
+                            ignored[ignore_glob] = []
+                        ignored[ignore_glob].append(subp)
+                    else:
+                        mutter("queue to add sub-file %r", subp)
+                        file_list.append(tree.abspath(subp))
+
+
+    mutter('added %d entries', len(added))
     
-    if count > 0:
+    if len(added) > 0:
         tree._write_inventory(inv)
 
-    return count
+    return added, ignored
 
 def __add_one(tree, inv, path, kind, action):
     """Add a file or directory, automatically add unversioned parents."""
@@ -174,10 +180,10 @@
     # This is safe from infinite recursion because the tree root is
     # always versioned.
     if inv.path2id(path) != None:
-        return 0
+        return []
 
     # add parent
-    count = __add_one(tree, inv, dirname(path), 'directory', action)
+    added = __add_one(tree, inv, dirname(path), 'directory', action)
     action(inv, path, kind)
 
-    return count + 1
+    return added + [path]

=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2005-12-15 03:21:11 +0000
+++ bzrlib/builtins.py	2005-12-15 08:32:03 +0000
@@ -222,9 +222,9 @@
     add them.
     """
     takes_args = ['file*']
-    takes_options = ['no-recurse', 'dry-run']
+    takes_options = ['no-recurse', 'dry-run', 'verbose']
 
-    def run(self, file_list, no_recurse=False, dry_run=False):
+    def run(self, file_list, no_recurse=False, dry_run=False, verbose=False):
         import bzrlib.add
 
         if dry_run:
@@ -238,7 +238,19 @@
         else:
             action = bzrlib.add.add_action_add_and_print
 
-        bzrlib.add.smart_add(file_list, not no_recurse, action)
+        added, ignored = bzrlib.add.smart_add(file_list, not no_recurse, 
+                                              action)
+        if len(ignored) > 0:
+            for glob in sorted(ignored.keys()):
+                match_len = len(ignored[glob])
+                if verbose:
+                    for path in ignored[glob]:
+                        print "ignored %s matching \"%s\"" % (path, glob)
+                else:
+                    print "ignored %d file(s) matching \"%s\"" % (match_len,
+                                                              glob)
+            print "If you wish to add some of these files, please add them"\
+                " by name."
 
 
 class cmd_mkdir(Command):
@@ -1772,7 +1784,7 @@
             show_ids=False, verbose=False):
         from bzrlib.missing import find_unmerged, iter_log_data
         from bzrlib.log import log_formatter
-        local_branch = bzrlib.branch.Branch.open_containing(".")[0]
+        local_branch = bzrlib.branch.Branch.open_containing(u".")[0]
         parent = local_branch.get_parent()
         if other_branch is None:
             other_branch = parent

=== modified file 'bzrlib/tests/__init__.py'
--- bzrlib/tests/__init__.py	2005-12-13 05:46:40 +0000
+++ bzrlib/tests/__init__.py	2005-12-13 18:56:57 +0000
@@ -264,6 +264,16 @@
             raise AssertionError('pattern "%s" not found in "%s"'
                     % (needle_re, haystack))
 
+    def AssertSubset(self, sublist, superlist):
+        """Assert that every entry in sublist is present in superlist."""
+        missing = []
+        for entry in sublist:
+            if entry not in superlist:
+                missing.append(entry)
+        if len(missing) > 0:
+            raise AssertionError("value(s) %r not present in container %r" % 
+                                 (missing, superlist))
+
     def _startLogFile(self):
         """Send bzr and test log messages to a temporary file.
 

=== modified file 'bzrlib/tests/blackbox/test_too_much.py'
--- bzrlib/tests/blackbox/test_too_much.py	2005-12-01 15:19:22 +0000
+++ bzrlib/tests/blackbox/test_too_much.py	2005-12-13 20:13:15 +0000
@@ -15,6 +15,10 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
+# Mr. Smoketoomuch: I'm sorry?
+# Mr. Bounder: You'd better cut down a little then.
+# Mr. Smoketoomuch: Oh, I see! Smoke too much so I'd better cut down a little
+#                   then!
 
 """Black-box tests for bzr.
 
@@ -663,13 +667,22 @@
     def test_add_reports(self):
         """add command prints the names of added files."""
         b = Branch.initialize('.')
-        self.build_tree(['top.txt', 'dir/', 'dir/sub.txt'])
+        self.build_tree(['top.txt', 'dir/', 'dir/sub.txt', 'CVS'])
         out = self.run_bzr_captured(['add'], retcode=0)[0]
         # the ordering is not defined at the moment
         results = sorted(out.rstrip('\n').split('\n'))
-        self.assertEquals(['added dir',
+        self.assertEquals(['If you wish to add some of these files, please'\
+                           ' add them by name.',
+                           'added dir',
                            'added dir'+os.sep+'sub.txt',
-                           'added top.txt',],
+                           'added top.txt',
+                           'ignored 1 file(s) matching "CVS"'],
+                          results)
+        out = self.run_bzr_captured(['add', '-v'], retcode=0)[0]
+        results = sorted(out.rstrip('\n').split('\n'))
+        self.assertEquals(['If you wish to add some of these files, please'\
+                           ' add them by name.',
+                           'ignored CVS matching "CVS"'],
                           results)
 
     def test_add_quiet_is(self):

=== modified file 'bzrlib/tests/test_smart_add.py'
--- bzrlib/tests/test_smart_add.py	2005-12-15 03:21:11 +0000
+++ bzrlib/tests/test_smart_add.py	2005-12-15 08:32:03 +0000
@@ -6,6 +6,7 @@
 from bzrlib.errors import NotBranchError, NoSuchFile
 from bzrlib.inventory import InventoryFile, Inventory
 from bzrlib.workingtree import WorkingTree
+from bzrlib.add import smart_add
 
 class TestSmartAdd(TestCaseInTempDir):
 
@@ -91,6 +92,19 @@
         branch = Branch.initialize(u".")
         self.assertRaises(NoSuchFile, smart_add, 'non-existant-file')
 
+    def test_returns(self):
+        """Correctly returns added/ignored files"""
+        from bzrlib.commands import run_bzr
+        b = Branch.initialize(u'.')
+        t = b.working_tree()
+        self.build_tree(['inertiatic/', 'inertiatic/esp', 'inertiatic/CVS', 
+                        'inertiatic/foo.pyc'])
+        added, ignored = smart_add(u'.')
+        self.AssertSubset(('inertiatic', 'inertiatic/esp'), added)
+        self.AssertSubset(('CVS', '*.py[oc]'), ignored)
+        self.AssertSubset(('inertiatic/CVS',), ignored['CVS'])
+        self.AssertSubset(('inertiatic/foo.pyc',), ignored['*.py[oc]'])
+
 
 class TestSmartAddBranch(TestCaseInTempDir):
     """Test smart adds with a specified branch."""
@@ -164,6 +178,7 @@
         for path in paths:
             self.assertNotEqual(tree.path2id(path), None)
 
+
 class TestAddActions(TestCaseInTempDir):
 
     def test_null(self):

